#!/usr/bin/env python
# -*- coding: utf-8 -*-
# Copyright (c) 2021 Carlos Ramos.

import unittest
from pprint import pprint
import sys
sys.path.insert(0, '../')
from linea import Linea


class TestLinea(unittest.TestCase):

    def test_imprime_linea_simple(self):
        """
        Imprime una línea sin modificadores. Es decir, texto que recibe, texto
        que se imprime.
        """

        linea = Linea("Esta es una línea simple")

        self.assertEqual("Esta es una línea simple", linea.texto_transformado)

    def test_reemplaza_inicio_de_gritos(self):
        """
        La línea contiene un símbolo para iniciar gritos, que se reemplaza por
        el HTML correspondiente.
        """

        linea = Linea("Esta línea abre |gritos")

        self.assertEqual(
            'Esta línea abre <span class="gritos">gritos',
            linea.texto_transformado
        )

    def test_reemplaza_inicio_y_final_de_gritos(self):
        """
        Dentro de una misma línea se abre y se cierra la modificación de voz a
        gritos.
        """

        linea = Linea("Aquí abre |gritos| y los cierra.")

        self.assertEqual(
            'Aquí abre <span class="gritos">gritos</span> y los cierra.',
            linea.texto_transformado
        )

    def test_reemplaza_final_de_gritos(self):
        """
        La línea contiene un símbolo para finalizar gritos, que se reemplaza
        por la etiqueta de cierra debido a que se inicializa el arreglo de
        pendientes con una barra vertical.
        """

        linea = Linea("fin de gritos| aquí", ["|"])

        self.assertEqual('fin de gritos</span> aquí', linea.texto_transformado)

    def test_elimina_espacios_en_blanco_de_final(self):
        """
        Imprime la línea eliminando los espacios del final.
        """

        linea = Linea("Espacios al final    ")

        self.assertEqual('Espacios al final', linea.texto_transformado)

    def test_elimina_espacios_en_blanco_de_inicio(self):
        """
        Imprime la línea eliminando los espacios del inicio.
        """

        linea = Linea("    Espacios al inicio")

        self.assertEqual('Espacios al inicio', linea.texto_transformado)

    def test_imprime_linea_con_comentario(self):
        """
        Imprime la parte del texto que no contiene comentario.
        """

        linea = Linea("Línea con % comentario")

        self.assertEqual("Línea con", linea.texto_transformado)

    def test_reemplaza_inicio_de_segunda_voz(self):
        """
        La línea contiene un símbolo de inicio de segunda voz, y se reemplaza
        por su HTML correspondiente.
        """

        linea = Linea("Inicia [segunda voz")

        self.assertEqual(
            'Inicia <span class="segunda-voz">segunda voz',
            linea.texto_transformado
        )

    def test_reemplaza_final_de_segunda_voz(self):
        """
        Reemplaza el final de la segunda voz, cuando un inicio de segunda voz
        le es enviado como voz activa.
        """

        linea = Linea("fin] segunda", ["["])

        self.assertEqual('fin</span> segunda', linea.texto_transformado)

    def test_reemplaza_inicio_y_final_de_segunda_voz(self):
        """
        Dentro de una misma línea se abre y se cierra la modificación a segunda
        voz.
        """

        linea = Linea("Segunda [voz] aquí.")

        self.assertEqual(
            'Segunda <span class="segunda-voz">voz</span> aquí.',
            linea.texto_transformado
        )

    def test_reemplaza_inicio_de_tercera_voz(self):
        """
        La línea contiene un símbolo de inicio de tercera voz, y se reemplaza
        por su HTML correspondiente.
        """

        linea = Linea("Inicia {tercera voz")

        self.assertEqual(
            'Inicia <span class="tercera-voz">tercera voz',
            linea.texto_transformado
        )

    def test_reemplaza_final_de_tercera_voz(self):
        """
        Reemplaza el final de la tercera voz, cuando un inicio de tercera voz
        le es enviado como voz activa.
        """

        linea = Linea("fin} tercera", ["{"])

        self.assertEqual('fin</span> tercera', linea.texto_transformado)

    def test_reemplaza_inicio_y_final_de_tercera_voz(self):
        """
        Dentro de una misma línea se abre y se cierra la modificación a tercera
        voz.
        """

        linea = Linea("Tercera {voz} aquí.")

        self.assertEqual(
            'Tercera <span class="tercera-voz">voz</span> aquí.',
            linea.texto_transformado
        )

    def test_las_estrellas_solas_se_imprimen(self):
        """
        Demuestra que el carácter de estrella no es necesario que se escape.
        """

        linea = Linea("La estrella * se imprime sin problemas.")

        self.assertEqual(
            "La estrella * se imprime sin problemas.",
            linea.texto_transformado
        )

    def test_cuenta_palabras(self):
        linea = Linea("Estas son cuatro palabras")

        self.assertEqual(linea.cantidad_de_palabras, 4)

    def test_cuenta_palabras_en_linea_con_comentario(self):
        linea = Linea("Estas son % cuatro palabras")

        self.assertEqual(linea.cantidad_de_palabras, 2)

    def test_cuenta_palabras_en_linea_con_porcentaje(self):
        linea = Linea("Esto es 10,000\% correcto")

        self.assertEqual(linea.cantidad_de_palabras, 4)

    def test_cuenta_palabras_en_linea_con_comentario_y_porcentaje(self):
        linea = Linea("Esto es 10,000\% correcto, pero % esto no")

        self.assertEqual(linea.cantidad_de_palabras, 5)

    ###########################################################################
    # FALTAN POR HACER ########################################################
    ###########################################################################

    def test_se_cierra_segunda_voz_sin_inicio(self):

        pass

    def test_se_cierra_tercera_voz_sin_inicio(self):

        pass


if __name__ == '__main__':
    unittest.main()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# Copyright (c) 2021 Carlos Ramos.

import unittest
from pprint import pprint
from ..letra import Letra
from .test_letra_base import TestLetraBase


class TestLetra(TestLetraBase):
    """
    Pruebas a nivel Letra.
    """

    def test_detecta_titulo(self):
        """
        La letra tiene título solamente, nada de contenido.
        """
        letra = self._letra_lee_archivo("titulo")

        self.assertEqual("Ejemplo de título", letra.titulo)
        self.assertEqual(0, len(letra.secciones))

    def test_detecta_titulo_desfasado(self):
        """
        La letra tiene título solamente, nada de contenido.
        """
        letra = self._letra_lee_archivo("titulo_desfasado")

        self.assertEqual("Ejemplo de título", letra.titulo)
        self.assertEqual(0, len(letra.secciones))

    def test_solo_versos(self):
        """
        La letra solo contiene versos, por lo que no se carga contenido.
        """

        letra = self._letra_lee_archivo("solo_versos")
        self.assertEqual(None, letra.titulo)

    def test_agrega_verso(self):
        """
        La letra tiene título y un verso.
        """
        letra = Letra()

        letra = self._letra_lee_archivo("titulo_y_verso")

        self.assertEqual(len(letra.secciones), 1)
        self.assertEqual(letra.secciones[0].tipo, "verso")

    def test_agrega_dos_versos(self):
        """
        La letra tiene título y dos versos.
        """
        letra = Letra()

        letra = self._letra_lee_archivo("titulo_y_dos_versos")

        self.assertEqual(len(letra.secciones), 2)
        self.assertEqual(letra.secciones[0].tipo, "verso")
        self.assertEqual(letra.secciones[1].tipo, "verso")

    def test_agrega_dos_versos_lineas_vacias(self):
        """
        La letra tiene título y dos versos.
        """
        letra = Letra()

        letra = self._letra_lee_archivo("titulo_y_dos_versos_lineas_vacias")

        self.assertEqual(len(letra.secciones), 2)
        self.assertEqual(letra.secciones[0].tipo, "verso")
        self.assertEqual(letra.secciones[1].tipo, "verso")

    def test_agrega_verso_y_coro(self):
        """
        Letra contiene verso y coro.
        """
        letra = Letra()

        letra = self._letra_lee_archivo("titulo_verso_y_coro")

        self.assertEqual(len(letra.secciones), 2)
        self.assertEqual(letra.secciones[0].tipo, "verso")
        self.assertEqual(letra.secciones[1].tipo, "coro")

    def test_agrega_intro_verso(self):
        """
        La letra tiene título, intro y un verso.
        """
        letra = Letra()

        letra = self._letra_lee_archivo("titulo_intro_verso")

        self.assertEqual(len(letra.secciones), 2)
        self._revisar_tipo_y_lineas(letra.secciones[0], "intro", 1)
        self._revisar_tipo_y_lineas(letra.secciones[1], "verso", 2)

    def test_agrega_verso_coro_y_puente(self):
        """
        Letra contiene verso, coro, y puente.
        """
        letra = Letra()

        letra = self._letra_lee_archivo("titulo_verso_coro_y_puente")

        self.assertEqual(len(letra.secciones), 3)
        self.assertEqual(letra.secciones[0].tipo, "verso")
        self.assertEqual(letra.secciones[1].tipo, "coro")
        self.assertEqual(letra.secciones[2].tipo, "puente")

    def test_agrega_coro_precoro_postcoro(self):
        """
        Letra contiene verso, coro, y puente.
        """
        letra = Letra()

        letra = self._letra_lee_archivo("titulo_coro_precoro_postcoro")

        self.assertEqual(len(letra.secciones), 3)
        self._revisar_tipo_y_lineas(letra.secciones[0], "precoro", 3)
        self._revisar_tipo_y_lineas(letra.secciones[1], "coro", 2)
        self._revisar_tipo_y_lineas(letra.secciones[2], "postcoro", 4)

    def test_agrega_intro_verso_outro(self):
        """
        Letra contiene verso, coro, y puente.
        """
        letra = Letra()

        letra = self._letra_lee_archivo("titulo_intro_verso_outro")

        self.assertEqual(len(letra.secciones), 3)
        self._revisar_tipo_y_lineas(letra.secciones[0], "intro", 3)
        self._revisar_tipo_y_lineas(letra.secciones[1], "verso", 4)
        self._revisar_tipo_y_lineas(letra.secciones[2], "outro", 1)

    def test_cuenta_lineas(self):

        letra = Letra()

        letra = self._letra_lee_archivo("titulo_coro_precoro_postcoro")

        self.assertEqual(letra.cantidad_de_lineas, 9)

    def test_cuenta_palabras(self):

        letra = Letra()

        letra = self._letra_lee_archivo("titulo_coro_precoro_postcoro")

        self.assertEqual(letra.cantidad_de_palabras, 18)

    ###########################################################################
    ### Funciones privadas para cargar los archivos de texto de las letras. ###
    ###########################################################################

    def _letra_lee_archivo(self, nombre_archivo):
        """
        Carga el archivo .me37 en un objeto Letra, mismo que es regresado tras
        leer dicho archivo.
        """

        letra = Letra()
        contenido = self._leer_archivo(nombre_archivo)

        return letra.leer(contenido)


if __name__ == '__main__':
    unittest.main()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# Copyright (c) 2021 Carlos Ramos.

import unittest
from pprint import pprint
import sys
from ..linea import Linea
from ..seccion import Seccion


class TestSeccion(unittest.TestCase):

    def test_seccion_convierte_cadena_a_linea(self):
        """
        Convierte una cadena de texto a un objeto Linea, y la agrega a las
        líneas dentro de la sección.
        """

        seccion = Seccion()
        seccion.agregar_linea("Esta es una línea simple")

        self.assertEqual(len(seccion.lineas), 1)
        self.assertEqual(
            "Esta es una línea simple",
            seccion.lineas[0].texto_original
        )

    def test_seccion_vacia_no_tiene_tipo(self):
        """
        Una sección vacía no tiene tipo todavía.
        """

        seccion = Seccion()

        self.assertEqual(None, seccion.tipo)

    def test_seccion_es_verso(self):
        """
        Una sección con una línea sin indicadores es tipo "verso"
        """

        seccion = Seccion()
        seccion.agregar_linea("Esta es una línea simple")

        self.assertEqual("verso", seccion.tipo)

    def test_seccion_es_coro(self):
        """
        Una sección con dos ** al inicio es un "coro".
        """

        seccion = Seccion()
        seccion.agregar_linea("**Primer línea de coro")

        self.assertEqual("coro", seccion.tipo)

    def test_seccion_es_puente(self):
        """
        Una sección con un _ al inicio es un "puente".
        """

        seccion = Seccion()
        seccion.agregar_linea("_Primer línea de puente")

        self.assertEqual("puente", seccion.tipo)

    def test_seccion_es_precoro(self):
        """
        Una sección con un ## al inicio es un "precoro".
        """

        seccion = Seccion()
        seccion.agregar_linea("##Primer línea de precoro")

        self.assertEqual("precoro", seccion.tipo)

    def test_seccion_es_postcoro(self):
        """
        Una sección con un ## al inicio es un "postcoro".
        """

        seccion = Seccion()
        seccion.agregar_linea("$$Primer línea de postcoro")

        self.assertEqual("postcoro", seccion.tipo)

    def test_seccion_es_intro(self):
        """
        Una sección con un _** al inicio es un "intro".
        """

        seccion = Seccion()
        seccion.agregar_linea("_**Primer línea de intro")

        self.assertEqual("intro", seccion.tipo)

    def test_seccion_es_outro(self):
        """
        Una sección con un ## al inicio es un "outro".
        """

        seccion = Seccion()
        seccion.agregar_linea("**_Primer línea de outro")

        self.assertEqual("outro", seccion.tipo)

    def test_bloque_de_coro_contiene_apertura(self):
        """
        Un bloque contiene la línea de apertura, una línea previa a las líneas.
        """
        seccion = Seccion()
        seccion.agregar_linea("**Primer línea de coro")

        self.assertTrue(len(seccion.bloque) > 1)
        self.assertIsInstance(seccion.bloque[0], str)
        self.assertEqual('<div class="seccion seccion-coro">', seccion.bloque[0])

    def test_primer_linea_de_verso_no_se_recorta(self):
        """
        Como el verso no contiene símbolos a ser recortados, esta prueba tiene
        como objetivo demostrar que el texto se mantiene íntegro.
        """

        seccion = Seccion()
        seccion.agregar_linea("Primera línea de verso")

        primera_linea = seccion.lineas[0].texto_transformado
        self.assertEqual('Primera línea de verso', primera_linea)

    def test_primer_linea_de_coro_no_contiene_simbolos(self):
        """
        La primera línea del coro no muestra los símbolos que indican que
        pertenece al coro.
        """

        seccion = Seccion()
        seccion.agregar_linea("**Primera línea de coro")

        primera_linea = seccion.lineas[0].texto_transformado
        self.assertEqual('Primera línea de coro', primera_linea)

    def test_primer_linea_de_puente_no_contiene_simbolos(self):
        """
        La primera línea del puente no muestra los símbolos que indican que
        pertenece al puente.
        """

        seccion = Seccion()
        seccion.agregar_linea("_Primera línea de puente")

        primera_linea = seccion.lineas[0].texto_transformado
        self.assertEqual('Primera línea de puente', primera_linea)

    def test_primer_linea_de_intro_no_contiene_simbolos(self):
        """
        La primera línea del intro no muestra los símbolos que indican que
        pertenece al intro.
        """

        seccion = Seccion()
        seccion.agregar_linea("_**Primera línea de intro")

        primera_linea = seccion.lineas[0].texto_transformado
        self.assertEqual('Primera línea de intro', primera_linea)

    def test_primer_linea_de_outro_no_contiene_simbolos(self):
        """
        La primera línea del outro no muestra los símbolos que indican que
        pertenece al outro.
        """

        seccion = Seccion()
        seccion.agregar_linea("**_Primera línea de outro")

        primera_linea = seccion.lineas[0].texto_transformado
        self.assertEqual('Primera línea de outro', primera_linea)

    def test_primer_linea_de_precoro_no_contiene_simbolos(self):
        """
        La primera línea del precoro no muestra los símbolos que indican que
        pertenece al precoro.
        """

        seccion = Seccion()
        seccion.agregar_linea("##Primera línea de precoro")

        primera_linea = seccion.lineas[0].texto_transformado
        self.assertEqual('Primera línea de precoro', primera_linea)

    def test_primer_linea_de_postcoro_no_contiene_simbolos(self):
        """
        La primera línea del postcoro no muestra los símbolos que indican que
        pertenece al postcoro.
        """

        seccion = Seccion()
        seccion.agregar_linea("$$Primera línea de postcoro")

        primera_linea = seccion.lineas[0].texto_transformado
        self.assertEqual('Primera línea de postcoro', primera_linea)

    def test_segunda_linea_de_coro_cierra_seccion(self):
        """
        Esta prueba demuestra que la última línea ya no imprime la clausura de
        la sección.
        """

        seccion = Seccion()
        seccion.agregar_linea("**Inicio")
        seccion.agregar_linea("Final**")

        inicio = seccion.lineas[0].texto_transformado
        self.assertEqual('Inicio', inicio)
        final = seccion.lineas[1].texto_transformado
        self.assertEqual('Final', final)

    def test_bloque_de_verso_tiene_etiqueta_html_de_cierre(self):

        seccion = Seccion()
        seccion.agregar_linea("Inicio")
        seccion.agregar_linea("Final")
        seccion.cerrar()

        self.assertEqual(4, len(seccion.bloque))
        self.assertEqual('<div class="seccion seccion-verso">', seccion.bloque[0])
        self.assertEqual("Inicio", seccion.bloque[1].texto_transformado)
        self.assertEqual("Final", seccion.bloque[2].texto_transformado)
        self.assertEqual("</div>", seccion.bloque[3])

    def test_bloque_de_coro_tiene_etiqueta_html_de_cierre(self):

        seccion = Seccion()
        seccion.agregar_linea("**Inicio")
        seccion.agregar_linea("Final**")

        self.assertEqual(4, len(seccion.bloque))
        self.assertEqual("</div>", seccion.bloque[3])

    def test_seccion_maneja_voz_en_dos_lineas(self):
        seccion = Seccion()
        seccion.agregar_linea("Aquí hay {otra")
        seccion.agregar_linea("voz} que acaba")
        seccion.cerrar()

        self.assertEqual(
            'Aquí hay <span class="tercera-voz">otra',
            seccion.lineas[0].texto_transformado
        )
        self.assertEqual(
            'voz</span> que acaba',
            seccion.lineas[1].texto_transformado
        )

    def test_seccion_cierra_gritos_sin_cierre(self):
        seccion = Seccion()
        seccion.agregar_linea("Gritos |abren")
        seccion.agregar_linea("Pero nunca cierran")

        self.assertEqual(
            'Gritos <span class="gritos">abren',
            seccion.bloque[1].texto_transformado
        )

        self.assertEqual(
            'Pero nunca cierran</span>',
            seccion.bloque[2].texto_transformado
        )

    def test_cantidad_de_lineas(self):

        seccion = Seccion()
        seccion.agregar_linea("**_Primera línea de outro")

        self.assertEqual(seccion.cantidad_de_lineas, 1)

    def test_cantidad_de_lineas_con_vacias(self):

        seccion = Seccion()
        seccion.agregar_linea("**_Primera línea de outro")
        seccion.agregar_linea("")
        seccion.agregar_linea("")
        seccion.agregar_linea("")
        seccion.agregar_linea("Una más**")

        self.assertEqual(seccion.cantidad_de_lineas, 2)

    def test_cantidad_de_palabras_una_linea(self):

        seccion = Seccion()
        seccion.agregar_linea("**_Primera línea de outro")

        self.assertEqual(seccion.cantidad_de_palabras, 4)

    def test_cantidad_de_palabras_dos_lineas(self):

        seccion = Seccion()
        seccion.agregar_linea("**_Primera línea de outro")
        seccion.agregar_linea("Segunda línea")

        self.assertEqual(seccion.cantidad_de_palabras, 6)

    def test_cantidad_de_palabras_con_lineas_vacias(self):

        seccion = Seccion()
        seccion.agregar_linea("**_Primera línea de outro")
        seccion.agregar_linea("")
        seccion.agregar_linea("")
        seccion.agregar_linea("Segunda línea")
        seccion.agregar_linea("")
        seccion.agregar_linea("")

        self.assertEqual(seccion.cantidad_de_palabras, 6)

    ###########################################################################
    # PENDIENTES ##############################################################
    ###########################################################################

    def test_seccion_cierra_voces(self):

        pass

    ###########################################################################
    # MECANISMO DE ADVERTENCIAS ###############################################
    ###########################################################################

    def test_se_crea_una_advertencia_por_agregar_linea_a_seccion_cerrada(self):

        pass


if __name__ == '__main__':
    unittest.main()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# Copyright (c) 2021 Carlos Ramos.

# TODO:
# + Reemplazo de comillas dobles.
# + Reemplazo de comillas simples.

from .linea import Linea
from .seccion import Seccion
from .letra import Letra

class LetraLaTeX(Letra):
    _diccionario = {
        "Letra": {
            "ContenedorInicio": "\\begin{letra}\n",
            "ContenedorFin": "\n\\end{letra}",
            "EspacioEntreSecciones": "\\ees\n",
        },
        "Titulo": {
            "Apertura": "\\titulo{",
            "Clausura": "}",
            "LineaPosterior": ""
        },
        "Linea": {
                "Apertura": {
                    Linea.GRITOS: "\\gritos{",
                    Linea.SEGUNDA_VOZ: "\\segundavoz{",
                    Linea.TERCERA_VOZ: "\\terceravoz{",
                },
                "Clausura": {
                    Linea.GRITOS_C: "}",
                    Linea.SEGUNDA_VOZ_C: "}",
                    Linea.TERCERA_VOZ_C: "}",
                },
                "SaltoDeLinea": "\\eel"
            },
        "Seccion": {
            "Apertura": {
                "[VERSO]": "\\begin{verso}",
                Seccion.INICIO_CORO: "\\begin{coro}",
                Seccion.INICIO_PUENTE: "\\begin{puente}",
                Seccion.INICIO_INTRO: "\\begin{intro}",
                Seccion.INICIO_OUTRO: "\\begin{outro}",
                Seccion.INICIO_PRECORO: "\\begin{precoro}",
                Seccion.INICIO_POSTCORO: "\\begin{postcoro}",
            },
            "Clausura": {
                "[/VERSO]": "\\end{verso}",
                Seccion.FIN_CORO: "\\end{coro}",
                Seccion.FIN_PUENTE: "\\end{puente}",
                Seccion.FIN_INTRO: "\\end{intro}",
                Seccion.FIN_OUTRO: "\\end{outro}",
                Seccion.FIN_PRECORO: "\\end{precoro}",
                Seccion.FIN_POSTCORO: "\\end{postcoro}",
            },
        }
    }


#!/usr/bin/env python
# -*- coding: utf-8 -*-
# Copyright (c) 2021 Carlos Ramos.

from marke37.linea import Linea
from marke37.seccion import Seccion
from marke37.letra import Letra

class LetraMarkdown(Letra):
    _diccionario = {
        "Titulo": {
            "Apertura": "# ",
            "Clausura": "",
        },
        "Linea": {
                "Apertura": {
                    Linea.GRITOS: "",
                    Linea.SEGUNDA_VOZ: "",
                    Linea.TERCERA_VOZ: "",
                },
                "Clausura": {
                    Linea.GRITOS_C: "",
                    Linea.SEGUNDA_VOZ_C: "",
                    Linea.TERCERA_VOZ_C: "",
                },
                "SaltoDeLinea": ""
            },
        "Seccion": {
            "Apertura": {
                "[VERSO]": "",
                Seccion.INICIO_CORO: "**",
                Seccion.INICIO_PUENTE: "_",
                Seccion.INICIO_INTRO: "_**",
                Seccion.INICIO_OUTRO: "**_",
                Seccion.INICIO_PRECORO: "**",
                Seccion.INICIO_POSTCORO: "**",
            },
            "Clausura": {
                "[/VERSO]": "",
                Seccion.FIN_CORO: "**",
                Seccion.FIN_PUENTE: "_",
                Seccion.FIN_INTRO: "**_",
                Seccion.FIN_OUTRO: "_**",
                Seccion.FIN_PRECORO: "**",
                Seccion.FIN_POSTCORO: "**",
            },
        }
    }


#!/usr/bin/env python
# -*- coding: utf-8 -*-
# Copyright (c) 2021 Carlos Ramos.

import os
from .letra import Letra
from .letra_latex import LetraLaTeX

class Marke37:
    """
    Esta clase transforma un archivo de Marke37 a diversos formatos.
    En caso de no especificar, el formato predeterminado es "html".

    Los formatos disponibles son:
        html   genera el html de la letra, solo contendedores y clases.
        pagina genera toda la página de html lista para ser mostrada en
               el navegador, utilizando una plantilla con HTML y CSS.
        latex  genera el código LaTeX con entornos e instrucciones que
               pueden ser definidas por el usuario, para su posterior
               compilación a PDF.
        pdf    esta opción genera el pdf de la letra, utilizando latex
               como intermedio.
    """

    def __init__(self):

        pass

    def convertir(self, archivo, formato="html"):
        letra = None

        if formato == "latex":
            letra = LetraLaTeX()

        if letra is None or formato == "html":
            letra = Letra()

        archivo = open(archivo, "r")
        texto_letra = archivo.read()

        for linea in texto_letra.split("\n"):
            letra.agregar_linea(linea)

        if formato == "pagina":
            return self._convertir_a_pagina(letra)

        return letra.imprimir()

    def _convertir_a_pagina(self, letra):
        """
        Esta conversión tiene implícita una conversión a HTML, que luego se
        reemplaza dentro de una página completa de HTML5, con hoja de estilo.
        """

        texto = ""

        nombre_plantilla = os.path.abspath(os.path.dirname(__file__)) + "/plantillas/html/bulma.html"

        archivo_plantilla = open(nombre_plantilla, "r")
        texto_plantilla = archivo_plantilla.read()

        texto = texto_plantilla.replace("{% titulo %}", letra.titulo)
        texto = texto.replace("{% bloque %}", letra.imprimir())

        return texto

    def _convertir_a_pdf(self, letra):
        """
        Esta conversión agrega el código suficiente para generar un
        documento autocontenido en PDF de la letra, tomando el código
        LaTeX como base.
        """

        pass



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# Copyright (c) 2021 Carlos Ramos.

import sys
import getopt
from typing import List
from os import path
from .marke37 import Marke37

# TODO: aceptar plantilla de página o pdf.

def _archivo_de_entrada_existe(archivo):
    if path.exists(archivo):
        return True
    else:
        return False

def main(argv: List[str] = sys.argv[1:]):
    archivo_me37 = None
    archivo_salida = None
    formato = "html"

    try:
        optlist, args = getopt.getopt(argv, "hi:o:f:")
    except getopt.GetoptError:
        print('me37 -i <archivo_me37> -f <formato> -o <archivo_convertido>')
        sys.exit(2)

    for opt, arg in optlist:
        if opt == '-h':
            print('me37 -i <archivo_me37> -f <formato> -o <archivo_convertido>')
            sys.exit()
        elif opt in ("-i"):
            archivo_me37 = arg
        elif opt in ("-f"):
            formato = arg
        elif opt in ("-o"):
            archivo_salida = arg

    if not _archivo_de_entrada_existe(archivo_me37):
        print("Archivo de entrada [{}] no encontrado...".format(archivo_me37))

    convertidor = Marke37()
    letra_convertida = convertidor.convertir(archivo_me37, formato)
    if not archivo_salida:
        print(letra_convertida)
    else:
        archivo_salida = open(archivo_salida, "w")
        archivo_salida.write(letra_convertida)


if __name__ == '__main__':
    sys.exit(main(sys.argv[1:]))



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# Copyright (c) 2021 Carlos Ramos.

import unittest

class TestLetraBase(unittest.TestCase):
    def _revisar_tipo_y_lineas(self, seccion, tipo, num_lineas):
        """
        Revisa el tipo de una sección.
        """

        self.assertEqual(seccion.tipo, tipo)
        self.assertEqual(len(seccion.lineas), num_lineas)

    def _leer_archivo(self, nombre_archivo):
        """
        Lee un archivo .me37 para usar como prueba.
        """
        letra = open("marke37/test/archivos/" + nombre_archivo + ".me37", "r")
        contenido = letra.read()
        letra.close()

        return contenido


#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# Copyright (c) 2021 Carlos Ramos.

import unittest

loader = unittest.TestLoader()
start_dir = './'
suite = loader.discover(start_dir)

runner = unittest.TextTestRunner()
runner.run(suite)


#!/usr/bin/env python
# -*- coding: utf-8 -*-
# Copyright (c) 2021 Carlos Ramos.

# TODO: Transformar a HTML, LaTeX, PDF, Markdown, reST, Messenger, y plano.

from marke37.seccion import Seccion

class Letra:
    _diccionario = None

    def __init__(self):

        self._titulo_provisional = None
        self._titulo = None

        self._seccion_actual = None

        self.secciones = []

        # Arreglar diccionario.
        if self._diccionario and "Linea" in self._diccionario:
            dicc = self._diccionario["Linea"]
            if "Seccion" not in self._diccionario:
                self._diccionario["Seccion"] = {}

            self._diccionario["Seccion"]["Linea"] = dicc

    def leer(self, letra):
        """
        Lee la letra completa, una cadena que proviene de un archivo.
        """

        for linea in letra.split('\n'):
            self.agregar_linea(linea)

        return self

    def agregar_linea(self, linea):
        """
        Agregar la línea, escogiendo la sección.
        """

        # Primero, descartamos las líneas que son comentario.
        if len(linea) > 0 and linea[0] == '%':
            return

        # Sigo buscando título, así que no entro en sección.
        if self._titulo is None:
            self._buscar_titulo(linea)
            return

        # Si es un salto de línea, lo usamos para cerrar sección.
        if len(linea.strip()) == 0:
            if self._hay_seccion_abierta():
                self._seccion_actual.cerrar()

            self._seccion_actual = None

            return

        # En este punto ya tenemos título y hemos eliminado los comentarios,
        # y ya se procesaron los saltos de línea.
        if self._seccion_actual is None:
            self._crear_seccion()

        self._seccion_actual.agregar_linea(linea)

    @property
    def titulo(self):
        """
        Regresa el título de la letra, sin adiciones de contenedor.
        """

        return self._titulo

    @property
    def cantidad_de_lineas(self):
        cantidad = 0

        for s in self.secciones:
            cantidad = cantidad + s.cantidad_de_lineas

        return cantidad

    @property
    def cantidad_de_palabras(self):
        cantidad = 0

        for s in self.secciones:
            cantidad = cantidad + s.cantidad_de_palabras

        return cantidad

    def imprimir(self):

        cadena = self._imprimir_contenedor_inicio()

        cadena += self._imprimir_titulo()

        primera_seccion = True
        for seccion in self.secciones:
            if primera_seccion:
                primera_seccion = False
            else:
                if self._diccionario and "Letra" in self._diccionario and "EspacioEntreSecciones" in self._diccionario["Letra"]:
                    cadena += self._diccionario["Letra"]["EspacioEntreSecciones"]

            if not seccion.esta_cerrada:
                seccion.cerrar()

            cadena += seccion.imprimir()

        cadena += self._imprimir_contenedor_fin()

        return cadena

    def _imprimir_contenedor_inicio(self):
        contenedor = ""

        if not self._diccionario:
            return contenedor

        if "Letra" not in self._diccionario:
            return contenedor

        if "ContenedorInicio" not in self._diccionario["Letra"]:
            return contenedor

        return self._diccionario["Letra"]["ContenedorInicio"] + "\n"

    def _imprimir_contenedor_fin(self):
        contenedor = ""

        if not self._diccionario:
            return contenedor

        if "Letra" not in self._diccionario:
            return contenedor

        if "ContenedorFin" not in self._diccionario["Letra"]:
            return contenedor

        return self._diccionario["Letra"]["ContenedorFin"] + "\n"

    def _imprimir_titulo(self):
        if not self._titulo:
            return ""

        titulo = '<h1 class="titulo">\n'
        titulo += self._titulo + "\n"
        titulo += "</h1>\n"

        if not self._diccionario:
            return titulo

        if "Titulo" not in self._diccionario:
            return titulo

        # Si ya se encontró el nodo de "Titulo", entonces proceder con las
        # sustituciones, sean las que sean.
        titulo = ""
        if "LineaAnterior" in self._diccionario["Titulo"]:
            titulo += self._diccionario["Titulo"]["LineaAnterior"] + "\n"

        if "Apertura" in self._diccionario["Titulo"]:
            titulo += self._diccionario["Titulo"]["Apertura"]

        titulo += self._titulo

        if "Clausura" in self._diccionario["Titulo"]:
            titulo += self._diccionario["Titulo"]["Clausura"]

        titulo += "\n"

        if "LineaPosterior" in self._diccionario["Titulo"]:
            titulo += self._diccionario["Titulo"]["LineaPosterior"] + "\n"

        return titulo

    def _crear_seccion(self):

        diccionario_de_seccion = None

        if self._diccionario and "Seccion" in self._diccionario:
            diccionario_de_seccion = self._diccionario["Seccion"]

        self._seccion_actual = Seccion(diccionario=diccionario_de_seccion)
        self.secciones.append(self._seccion_actual)

    def _hay_seccion_abierta(self):
        if self._seccion_actual is None:
            return False

        return not self._seccion_actual.esta_cerrada


    def _buscar_titulo(self, linea):
        if linea.startswith("==="):
            self._titulo = self._titulo_provisional
            return

        self._titulo_provisional = linea



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# Copyright (c) 2021 Carlos Ramos.

import unittest
from pprint import pprint
from ..letra_im import LetraIM
from .test_letra_base import TestLetraBase

class TestLetraIM(TestLetraBase):
    def test_letra_im_se_puede_instanciar(self):
        letra = LetraIM()

        self.assertIsInstance(letra, LetraIM)

    def test_letra_im_imprime_titulo(self):
        """
        Revisamos que se imprima verso en formato IM
        """

        letra = self._letra_lee_archivo("titulo_y_verso")

        self.assertEqual(len(letra.secciones), 1)

        s = letra.secciones[0]
        self.assertEqual(s.tipo, "verso")
        self.assertEqual(len(s.bloque), 5)
        self.assertEqual(s.bloque[1].imprimir(), "Primer renglón")
        self.assertEqual(s.bloque[2].imprimir(), "Segundo renglón")
        self.assertEqual(s.bloque[3].imprimir(), "Tercer renglón")
        self.assertEqual(s.bloque[4].imprimir(), "Cuarto renglón")

    def test_letra_im_imprime_verso_y_coro(self):
        """
        Revisamos la impresión correcta de verso y coro
        """

        letra = self._letra_lee_archivo("titulo_verso_y_coro")

        self.assertEqual(len(letra.secciones), 2)

        s = letra.secciones[0]
        self.assertEqual(s.tipo, "verso")
        self.assertEqual(len(s.bloque), 5)
        self.assertEqual(s.bloque[1].imprimir(), "Primer renglón")
        self.assertEqual(s.bloque[2].imprimir(), "Segundo renglón")
        self.assertEqual(s.bloque[3].imprimir(), "Tercer renglón")
        self.assertEqual(s.bloque[4].imprimir(), "Cuarto renglón")

        s = letra.secciones[1]
        self.assertEqual(s.tipo, "coro")
        self.assertEqual(len(s.bloque), 5)
        self.assertEqual(s.bloque[1].imprimir(), "*Coro 1*")
        self.assertEqual(s.bloque[2].imprimir(), "*Coro 2*")
        self.assertEqual(s.bloque[3].imprimir(), "*Coro 3*")
        self.assertEqual(s.bloque[4].imprimir(), "*Coro 4*")

    def test_letra_im_imprime_verso_coro_y_puente(self):
        """
        Letra imprime verso, coro, y puente.
        """

        letra = self._letra_lee_archivo("titulo_verso_coro_y_puente")

        s = letra.secciones[0]
        self.assertEqual(s.tipo, "verso")
        self.assertEqual(s.bloque[1].imprimir(), "Primer renglón")

        s = letra.secciones[1]
        self.assertEqual(s.tipo, "coro")
        self.assertEqual(s.bloque[1].imprimir(), "*Coro 1*")

        s = letra.secciones[2]
        self.assertEqual(s.tipo, "puente")
        self.assertEqual(s.bloque[1].imprimir(), "_Puente 1_")

    ###########################################################################
    ### Funciones privadas para cargar los archivos de texto de las letras. ###
    ###########################################################################

    def _letra_lee_archivo(self, nombre_archivo):
        """
        Carga el archivo .me37 en un objeto Letra, mismo que es regresado tras
        leer dicho archivo.
        """

        letra = LetraIM()
        contenido = self._leer_archivo(nombre_archivo)

        return letra.leer(contenido)


if __name__ == '__main__':
    unittest.main()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# Copyright (c) 2021 Carlos Ramos.

from .linea import Linea
from .seccion import Seccion
from .letra import Letra

class LetraIM_HTML(Letra):
    _diccionario = {
        "Letra": {
            "ContenedorInicio": '<div class="letra-im-html">',
            "ContenedorFin": '</div>',
            "EspacioEntreSecciones": "<br>\n",
        },
        "Titulo": {
            "Apertura": '<h1 class="titulo">```',
            "Clausura": '```</h1>',
            "LineaPosterior": "<br>"
        },
        "Linea": {
                "Apertura": {
                    Linea.GRITOS: '<span class="gritos">',
                    Linea.SEGUNDA_VOZ: '<span class="segunda-voz">',
                    Linea.TERCERA_VOZ: '<span class="tercera-voz">',
                },
                "Clausura": {
                    Linea.GRITOS_C: '</span>',
                    Linea.SEGUNDA_VOZ_C: '</span>',
                    Linea.TERCERA_VOZ_C: '</span>',
                },
                "SaltoDeLinea": "<br>"
            },
        "Seccion": {
            "Apertura": {
                "[VERSO]": '<div class="seccion seccion-verso">',
                Seccion.INICIO_CORO: '<div class="seccion seccion-coro">',
                Seccion.INICIO_PUENTE: '<div class="seccion seccion-puente">',
                Seccion.INICIO_INTRO: '<div class="seccion seccion-intro">',
                Seccion.INICIO_OUTRO: '<div class="seccion seccion-outro">',
                Seccion.INICIO_PRECORO: '<div class="seccion seccion-precoro">',
                Seccion.INICIO_POSTCORO: '<div class="seccion seccion-postcoro">',
                "LineaVerso": "",
                "LineaCoro": "*",
                "LineaPrecoro": "*",
                "LineaPostcoro": "*",
                "LineaPuente": "_",
                "LineaIntro": "_*",
                "LineaOutro": "*_",
            },
            "Clausura": {
                "[/VERSO]": '</div>',
                Seccion.FIN_CORO: '</div>',
                Seccion.FIN_PUENTE: '</div>',
                Seccion.FIN_INTRO: '</div>',
                Seccion.FIN_OUTRO: '</div>',
                Seccion.FIN_PRECORO: '</div>',
                Seccion.FIN_POSTCORO: '</div>',
                "LineaVerso": "",
                "LineaCoro": "*",
                "LineaPrecoro": "*",
                "LineaPostcoro": "*",
                "LineaPuente": "_",
                "LineaIntro": "*_",
                "LineaOutro": "_*",
            },
        }
    }


#!/usr/bin/env python
# -*- coding: utf-8 -*-
# Copyright (c) 2021 Carlos Ramos.

# TODO: Contar líneas

from .linea import Linea


class Seccion:
    INICIO_CORO = "**"
    INICIO_PUENTE = "_"
    INICIO_INTRO = "_**"
    INICIO_OUTRO = "**_"
    INICIO_PRECORO = "##"
    INICIO_POSTCORO = "$$"

    FIN_CORO = "**"
    FIN_PUENTE = "_"
    FIN_INTRO = "**_"
    FIN_OUTRO = "_**"
    FIN_PRECORO = "##"
    FIN_POSTCORO = "$$"

    SECCIONES = {
        INICIO_OUTRO: FIN_OUTRO,
        INICIO_INTRO: FIN_INTRO,
        INICIO_CORO: FIN_CORO,
        INICIO_PRECORO: FIN_PRECORO,
        INICIO_POSTCORO: FIN_POSTCORO,
        INICIO_PUENTE: FIN_PUENTE
    }

    CLASES = {
        "[VERSO]": "verso",
        INICIO_CORO: "coro",
        INICIO_PUENTE: "puente",
        INICIO_INTRO: "intro",
        INICIO_OUTRO: "outro",
        INICIO_PRECORO: "precoro",
        INICIO_POSTCORO: "postcoro"
    }

    APERTURA_DE_SECCION = {
        "[VERSO]": '<div class="seccion seccion-verso">',
        INICIO_CORO: '<div class="seccion seccion-coro">',
        INICIO_PUENTE: '<div class="seccion seccion-puente">',
        INICIO_INTRO: '<div class="seccion seccion-intro">',
        INICIO_OUTRO: '<div class="seccion seccion-outro">',
        INICIO_PRECORO: '<div class="seccion seccion-precoro">',
        INICIO_POSTCORO: '<div class="seccion seccion-postcoro">'
    }

    CLAUSURA_DE_SECCION = {
        "[/VERSO]": '</div>',
        FIN_CORO: '</div>',
        FIN_PUENTE: '</div>',
        FIN_INTRO: '</div>',
        FIN_OUTRO: '</div>',
        FIN_PRECORO: '</div>',
        FIN_POSTCORO: '</div>'
    }

    def __init__(self, apertura=None, diccionario=None):
        """
        Inicializa la lista de voces que contiene la sección.
        """

        # Inicializa la lista de líneas.
        self.lineas = []
        # Inicializa la lista de voces en la letra.
        self.voces = []
        # Recibo la apertura aquí, posiblemente desde la letra. Si no es así,
        # lo establecemos a "None" para evitar cargar el tipo u otros datos.
        self._apertura = apertura
        # Determina si ya se recibió o no el elemento que indica que se cerró
        # la sección.
        self._esta_cerrada = False
        # Contiene el diccionario que la sección le pasa a cada una de las
        # líneas que pertenecen a ella.
        self._diccionario_de_linea = None

        # Cadenas de sustitución
        self._diccionario = diccionario

    def agregar_linea(self, linea):
        """
        Agrega una línea a la sección.

        :param linea: Línea de la letra a ser agregada a la sección.
        """

        # Revisar la línea por apertura.
        if len(self.lineas) <= 0:
            apertura = self._buscar_apertura(linea)
            if apertura:
                self._apertura = apertura
                linea = self._recortar_apertura_de_linea(linea)

        # Revisar la línea por clausura.
        clausura = self._buscar_clausura(linea)
        if clausura:
            linea = self._recortar_clausura_de_linea(linea)
            self.cerrar()

        # Crea un objeto línea en base a la cadena, pasando el diccionario de
        # línea en base a lo que recibe la sección.
        objLinea = Linea(linea, self.voces, self.diccionario_de_linea)
        self.lineas.append(objLinea)

    def cerrar(self):
        """
        Cierra la sección, estableciendo la bandera de cierre.
        """

        self._esta_cerrada = True

    @property
    def cantidad_de_lineas(self):
        cantidad = 0

        for l in self.lineas:
            if len(l.texto_transformado) > 0:
                cantidad = cantidad + 1

        return cantidad

    @property
    def cantidad_de_palabras(self):
        cantidad = 0

        for l in self.lineas:
            cantidad = cantidad + l.cantidad_de_palabras

        return cantidad

    @property
    def tipo(self):
        """
        Regresa el tipo de sección en base a los primeros tres caracteres de la
        primera línea.
        """

        # Si no estableción el tipo, ni tenemos líneas todavía, no podemos
        # decir qué tipo de sección es.
        if self._apertura is None:
            return None

        return self.CLASES[self._apertura]

    @property
    def bloque(self):
        """
        Regresa el conjunto de líneas de la sección, más los indicadores de
        sección, como un arreglo de líneas.
        """

        self._bloque = []

        # Revisamos si la sección se abre con una línea previa.
        if self._apertura in self.APERTURA_DE_SECCION:
            # En el caso de que sí se deba imprimir algo en la línea previa,
            # se agrega al bloque. En caso contrario, se omite.
            if self._linea_previa() is not None:
                self._bloque.append(self._linea_previa())

        # Se agrega para que la última línea cierre las voces que no se
        # cerraron con marcado adecuado.
        if len(self.lineas) > 0:
            self.lineas[-1]._cerrar_voces = True

        # Agregamos cada línea de la sección.
        for linea in self.lineas:
            self._bloque.append(linea)

        # Si la sección ya está cerrada, entonces podemos agregar el cierre.
        if self._esta_cerrada:
            if self._linea_posterior() is not None:
                # En caso de que no se requieran etiquetas de clausura de
                # sección, no se agregan al bloque.
                self._bloque.append(self._linea_posterior())

        return self._bloque

    @property
    def es_verso(self):
        """
        Bandera para saber si la sección actual es verso o no.
        """

        self._es_verso = False

        if self._apertura == "[VERSO]":
            self._es_verso = True

        return self._es_verso

    @property
    def esta_cerrada(self):
        return self._esta_cerrada

    def _buscar_apertura(self, linea):
        """
        Busca el indicador de apertura en la línea provista.

        :param linea: Cadena en la que se va a buscar el indicador de apertura.
        """

        if isinstance(linea, str):
            texto_de_linea = linea
        else:
            texto_de_linea = linea.texto_original

        for apertura in self.SECCIONES.keys():
            if apertura == texto_de_linea[0:len(apertura)]:
                return apertura

        return "[VERSO]"

    def _texto_de_inicio_de_linea(self):
        """
        Obtiene el texto a agregar al inicio de cada línea de la sección.
        """

        if not self._diccionario:
            return ""

        if "Apertura" not in self._diccionario:
            return ""

        inicio_linea = "Linea" + self.tipo.title()
        if inicio_linea not in self._diccionario["Apertura"]:
            return ""

        return self._diccionario["Apertura"][inicio_linea]

    def _texto_de_final_de_linea(self):
        """
        Obtiene el texto a agregar al final de cada línea de la sección.
        """

        if not self._diccionario:
            return ""

        if "Clausura" not in self._diccionario:
            return ""

        fin_linea = "Linea" + self.tipo.title()
        if fin_linea not in self._diccionario["Clausura"]:
            return ""

        return self._diccionario["Clausura"][fin_linea]

    def _buscar_clausura(self, linea):
        """
        Busca el indicador de clausura en la línea provista.

        :param linea: Cadena en la que se va a buscar el indicador de clausura.
        """

        if self._apertura == "[VERSO]":
            return None

        if isinstance(linea, str):
            texto_de_linea = linea
        else:
            texto_de_linea = linea.texto_original

        clausura = self.SECCIONES[self._apertura]
        if clausura == texto_de_linea[-len(clausura):]:
            return clausura

        return None

    def _recortar_apertura_de_linea(self, linea):
        """
        Recorta el indicador de apertura de sección de la línea que recibe como
        argumento.
        En sí, evita el corte en caso de no tener sección definida, o si la
        sección en cuestión es un verso.
        """

        if self._apertura is None or self._apertura == "[VERSO]":
            return linea

        return linea[len(self._apertura):]

    def _recortar_clausura_de_linea(self, linea):
        """
        Recorta el indicador de clausura de sección de la línea que recibe como
        argumento.
        Este corte solamente aplica para el tipo de sección ya abierta. Es
        decir, un puente busca el símbolo de cierre de puente, y un coro busca
        cerrar un coro.
        De momento, no se plantea hacer nada si la sección es una pero el
        cierre corresponde a otra.
        """

        # Si no hay indicador, no se hace nada.
        if self._apertura is None or self._apertura == "[VERSO]":
            return linea

        # Si lo único que hay en la línea es potencialmente el indicador,
        # tampoco se hace nada.
        clausura = self.SECCIONES[self._apertura]
        if len(linea) <= len(self._apertura):
            return linea

        if linea[-len(self._apertura):] == clausura:
            return linea[0:-len(clausura)]

        return linea[len(self._apertura):]

    def imprimir(self):
        """
        Regresa el bloque convertido en cadena.
        """

        cadena = ""

        for linea in self.bloque:
            if isinstance(linea, str):
                cadena += linea
            else:
                cadena += linea.imprimir()
            cadena += "\n"

        return cadena

    def _linea_previa(self):
        """
        Regresa el texto que va una línea previo a la primera línea de la
        sección.
        """
        texto = self.APERTURA_DE_SECCION[self._apertura]

        if self._diccionario is None:
            return texto

        if "Apertura" not in self._diccionario:
            return texto

        if self._apertura not in self._diccionario["Apertura"]:
            return texto

        return self._diccionario["Apertura"][self._apertura]

    def _linea_posterior(self):
        """
        Regresa el texto que va en la línea siguiente a la última línea de la
        sección.
        """

        if self.es_verso:
            clausura = "[/VERSO]"
        else:
            clausura = self.SECCIONES[self._apertura]

        texto = self.CLAUSURA_DE_SECCION[clausura]

        if self._diccionario is None:
            return texto

        if "Clausura" not in self._diccionario:
            return texto

        if clausura not in self._diccionario["Clausura"]:
            return texto

        return self._diccionario["Clausura"][clausura]

    @property
    def diccionario_de_linea(self):
        """
        Crea y almacena el diccionario a ser enviado a cada línea.
        """

        if self._diccionario_de_linea is not None:
            return self._diccionario_de_linea

        if self._diccionario and "Linea" in self._diccionario:
            # Hacer una copia es necesaria, de lo contrario cada sección se
            # imprimirá con los símbolos de la última sección agregada.
            diccionario_de_linea = self._diccionario["Linea"].copy()
        else:
            diccionario_de_linea = {}

        diccionario_de_linea["Inicio"] = self._texto_de_inicio_de_linea()
        diccionario_de_linea["Final"] = self._texto_de_final_de_linea()

        self._diccionario_de_linea = diccionario_de_linea

        return self._diccionario_de_linea


#!/usr/bin/env python
# -*- coding: utf-8 -*-
# Copyright (c) 2021 Carlos Ramos.

import unittest
from pprint import pprint
from ..letra_latex import LetraLaTeX
from .test_letra_base import TestLetraBase


class TestLetraLaTeX(TestLetraBase):
    def test_letra_latex_se_puede_instanciar(self):
        letra = LetraLaTeX()

        self.assertTrue(1)

    def test_con_verso(self):
        letra = LetraLaTeX()

        letra.agregar_linea("Título")
        letra.agregar_linea("======")

        letra.agregar_linea("Verso")

        self.assertTrue(1)

    def test_con_gritos(self):
        letra = LetraLaTeX()

        letra.agregar_linea("Título")
        letra.agregar_linea("======")

        letra.agregar_linea("Verso |gritando| mucho.")

    def test_con_verso(self):
        letra = LetraLaTeX()

        letra.agregar_linea("Título")
        letra.agregar_linea("======")

        letra.agregar_linea("Verso")
        letra.agregar_linea("")
        letra.agregar_linea("**Coro")

        self.assertTrue(1)


if __name__ == '__main__':
    unittest.main()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# Copyright (c) 2021 Carlos Ramos.

# TODO: Contar palabras

class Linea:
    """
    Sirve para procesar una línea de una letra de canción en formato Marke37,
    para su transformación en diversos formatos, como HTML, reStructuredText,
    LaTeX, y PDF.
    """
    SIMBOLO_COMENTARIO = "%"

    # Símbolos para denotar la apertura de un cambio de voz.
    GRITOS = "|"
    SEGUNDA_VOZ = "["
    TERCERA_VOZ = "{"
    # Símbolos para denotar la clausura del cambio de voz.
    GRITOS_C = "|"
    SEGUNDA_VOZ_C = "]"
    TERCERA_VOZ_C = "}"

    # Relación de los símbolos de apertura con su cierre.
    VOCES = {
        GRITOS:      GRITOS_C,
        SEGUNDA_VOZ: SEGUNDA_VOZ_C,
        TERCERA_VOZ: TERCERA_VOZ_C,
    }

    # Relación de sustitución del marcado de apertura.
    APERTURA_DE_VOZ = {
        GRITOS:      '<span class="gritos">',
        SEGUNDA_VOZ: '<span class="segunda-voz">',
        TERCERA_VOZ: '<span class="tercera-voz">',
    }

    # Relación de sustitución del marcado de clausura.
    CLAUSURA_DE_VOZ = {
        GRITOS_C: '</span>',
        SEGUNDA_VOZ_C: '</span>',
        TERCERA_VOZ_C: '</span>',
    }

    def __init__(self, texto, efectos_abiertos=None, diccionario=None, cerrar_voces=False):
        self.texto_original = texto
        self.texto_plano = ""

        if efectos_abiertos is None:
            self.efectos_abiertos = []
        else:
            self.efectos_abiertos = efectos_abiertos

        self._cerrar_voces = cerrar_voces

        self._diccionario = diccionario

    def imprimir(self, cerrar_voces=False):
        """
        Imprime la cadena transformada.

        :returns: Texto procesado en el formato requerido.
        :rtype: string
        """

        linea = self._imprimir_antes_de()
        linea += self.texto_transformado
        linea += self._imprimir_despues_de()
        linea += self._imprimir_salto_de_linea()

        return linea

    @property
    def texto_transformado(self):
        texto = self._texto_sin_comentario()

        # Si se recibe la instrucción de cerrar voces, se forzan las voces
        # abiertas como símbolos al final del texto.
        if self._cerrar_voces:
            if len(self.efectos_abiertos) > 0:
                for efecto in self.efectos_abiertos:
                    texto += self.VOCES[efecto]

        texto = self._reemplazar_voces(texto, self.efectos_abiertos)

        self._texto_transformado = texto

        return self._texto_transformado

    @property
    def cantidad_de_palabras(self):
        texto = self._texto_sin_comentario()

        self._cantidad_de_palabras = len(texto.split())

        return self._cantidad_de_palabras

    def _texto_sin_comentario(self):
        """
        Elimina los comentarios de la línea recibida.
        """

        texto = self.texto_original.strip()

        # Si no hay signo de '%', entonces regresar todo el texto.
        if not self.SIMBOLO_COMENTARIO in texto:
            return texto

        # Si la línea es enteramente comentario, regresamos una cadena vacía.
        if texto[0] == self.SIMBOLO_COMENTARIO:
            return ''

        # Si no es completamente comentario, hay que analizar si el porciento debe ser texto.
        indices = [i for i, letra in enumerate(self.texto_original) if letra == self.SIMBOLO_COMENTARIO]

        indices_a_eliminar = []
        for i in indices:
            # Si tenemos una diagonal invertida, entonces es texto legítimo,
            # y lo dejamos en paz.
            if texto[i-1] == '\\':
                indices_a_eliminar.append(i - 1)
                continue

            # En este punto, ya sabemos que no es comentario. Regresamos el texto hasta esta posición.
            texto_l = list(texto)
            diagonalesCnt = len(indices_a_eliminar)
            print(indices_a_eliminar)
            if diagonalesCnt > 0:
                indices_a_eliminar.reverse() # Invertir para no tener que recorrer.
                for j in indices_a_eliminar:
                    texto_l.pop(j)

            return ''.join(texto_l[0:i - diagonalesCnt])

        return texto

    def _reemplazar_voces(self, texto, voces):
        """
        En esta función es donde se procesa la cadena de datos
        que proviene de la sección.
        """

        texto_procesado = ""
        for c in texto:
            # Primero buscamos si tenemos efectos abiertos...
            if len(voces) > 0:
                # Para buscar cerrar el último en la lista.
                texto_procesado += self._cerrar_voz(c, voces)
            else:
                texto_procesado += self._abrir_voz(c, voces)

        # Se corre con strip para eliminar espacios dejados por un comentario
        # que se eliminó en el preocesamiento.
        return texto_procesado.strip()

    def _cerrar_voz(self, caracter, voces):
        """
        Busca los símbolos de clausura, para ver si coinciden con el carácter
        actual.
        """

        reemplazo = caracter

        ultimo_efecto = voces[-1]
        simbolo_clausura = self.VOCES[ultimo_efecto]
        if caracter == simbolo_clausura:
            voces.pop()
            reemplazo = self.CLAUSURA_DE_VOZ[simbolo_clausura]

            # Para obtener el reemplazo ahora se busca en el diccionario.
            if not self._diccionario:
                return reemplazo

            if "Clausura" not in self._diccionario:
                return reemplazo

            if simbolo_clausura not in self._diccionario["Clausura"]:
                return reemplazo

            return self._diccionario["Clausura"][simbolo_clausura]

        return reemplazo

    def _abrir_voz(self, caracter, voces):
        """
        Busca dentro de los caracteres que definen las voces para ver si se
        va a abrir una nueva en base al caracter actual.
        """

        reemplazo = caracter

        for simbolo in self.VOCES.keys():
            if caracter == simbolo:
                voces.append(simbolo)
                # El reemplazo predeterminado...
                reemplazo = self.APERTURA_DE_VOZ[simbolo]

                # Para obtener el reemplazo ahora se busca en el diccionario.
                if not self._diccionario:
                    return reemplazo

                if "Apertura" not in self._diccionario:
                    return reemplazo

                if simbolo not in self._diccionario["Apertura"]:
                    return reemplazo

                return self._diccionario["Apertura"][simbolo]

        return reemplazo

    ###########################################################################
    # Funciones de impresión ##################################################
    ###########################################################################

    def _imprimir_antes_de(self):
        """
        Regresa una cadena que se imprime antes del texto de la letra. Está
        diseñada para ser reescrita por implementaciones de diversos formatos,
        para agregar el HTML, reST, o LaTeX necesario, según sea el caso.
        """

        if self._diccionario is None:
            return ''

        if "Inicio" not in self._diccionario:
            return ''

        return self._diccionario['Inicio']

    def _imprimir_despues_de(self):
        """
        Regresa una cadena con el contenido que se imprime después de la línea,
        que sirve para cerrar etiquetas que aún no se cierran (que se deben
        volver a abrir en el próximo renglón).
        """

        if self._diccionario is None:
            return ''

        if "Final" not in self._diccionario:
            return ''

        return self._diccionario['Final']

    def _imprimir_salto_de_linea(self):
        """
        Regresa una candena que representa un salto de línea, y se imprime
        después del contenido que va después del texto de la línea, y cualquier
        marcado que se necesite para acabar.
        """

        salto = '<br>'

        if self._diccionario is None:
            return salto

        if "SaltoDeLinea" not in self._diccionario:
            return salto

        return self._diccionario["SaltoDeLinea"]



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# Copyright (c) 2021 Carlos Ramos.

import unittest
from pprint import pprint
from ..letra_md import LetraMarkdown
from .test_letra_base import TestLetraBase

class TestLetraMarkdown(TestLetraBase):
    def test_letra_md_se_puede_instanciar(self):
        letra = LetraMarkdown()

        self.assertIsInstance(letra, LetraMarkdown)

    def test_letra_md_imprime_titulo(self):
        """
        Revisamos que se imprima verso en formato Markdown
        """

        letra = self._letra_lee_archivo("titulo_y_verso")

        self.assertEqual(len(letra.secciones), 1)

    def test_letra_md_imprime_coro(self):
        """
        Revisamos que se imprima coro en formato Markdown
        """

        letra = self._letra_lee_archivo("titulo_verso_y_coro")

        self.assertEqual(len(letra.secciones), 2)

        print(letra.imprimir())

    # def test_letra_md_imprime_titulo(self):
    #     """
    #     Revisamos que se imprima verso en formato Markdown
    #     """

    #     letra = self._letra_lee_archivo("titulo_y_verso")

    #     self.assertEqual(len(letra.secciones), 1)

    #     s = letra.secciones[0]
    #     self.assertEqual(s.tipo, "verso")
    #     self.assertEqual(len(s.bloque), 5)
    #     self.assertEqual(s.bloque[1].imprimir(), "Primer renglón")
    #     self.assertEqual(s.bloque[2].imprimir(), "Segundo renglón")
    #     self.assertEqual(s.bloque[3].imprimir(), "Tercer renglón")
    #     self.assertEqual(s.bloque[4].imprimir(), "Cuarto renglón")

    # def test_letra_md_imprime_verso_y_coro(self):
    #     """
    #     Revisamos la impresión correcta de verso y coro
    #     """

    #     letra = self._letra_lee_archivo("titulo_verso_y_coro")

    #     self.assertEqual(len(letra.secciones), 2)

    #     s = letra.secciones[0]
    #     self.assertEqual(s.tipo, "verso")
    #     self.assertEqual(len(s.bloque), 5)
    #     self.assertEqual(s.bloque[1].imprimir(), "Primer renglón")
    #     self.assertEqual(s.bloque[2].imprimir(), "Segundo renglón")
    #     self.assertEqual(s.bloque[3].imprimir(), "Tercer renglón")
    #     self.assertEqual(s.bloque[4].imprimir(), "Cuarto renglón")

    #     s = letra.secciones[1]
    #     self.assertEqual(s.tipo, "coro")
    #     self.assertEqual(len(s.bloque), 5)
    #     self.assertEqual(s.bloque[1].imprimir(), "*Coro 1*")
    #     self.assertEqual(s.bloque[2].imprimir(), "*Coro 2*")
    #     self.assertEqual(s.bloque[3].imprimir(), "*Coro 3*")
    #     self.assertEqual(s.bloque[4].imprimir(), "*Coro 4*")

    # def test_letra_md_imprime_verso_coro_y_puente(self):
    #     """
    #     Letra imprime verso, coro, y puente.
    #     """

    #     letra = self._letra_lee_archivo("titulo_verso_coro_y_puente")

    #     s = letra.secciones[0]
    #     self.assertEqual(s.tipo, "verso")
    #     self.assertEqual(s.bloque[1].imprimir(), "Primer renglón")

    #     s = letra.secciones[1]
    #     self.assertEqual(s.tipo, "coro")
    #     self.assertEqual(s.bloque[1].imprimir(), "*Coro 1*")

    #     s = letra.secciones[2]
    #     self.assertEqual(s.tipo, "puente")
    #     self.assertEqual(s.bloque[1].imprimir(), "_Puente 1_")

    ###########################################################################
    ### Funciones privadas para cargar los archivos de texto de las letras. ###
    ###########################################################################

    def _letra_lee_archivo(self, nombre_archivo):
        """
        Carga el archivo .me37 en un objeto Letra, mismo que es regresado tras
        leer dicho archivo.
        """

        letra = LetraMarkdown()
        contenido = self._leer_archivo(nombre_archivo)

        return letra.leer(contenido)


if __name__ == '__main__':
    unittest.main()
